import processing.video.*;    //WebCam Bib

Capture video;    // Definition aller Variablen, Vektoren und Objekte

PImage prevFrame;

float threshold = 50;
int Mx = 0;
int My = 0;
int ave = 0;

int ballX = width/2;
int ballY = height/2;
int rsp = 25;


float inc = 0.1;
int scl = 10;
float zoff = 0;

int cols;
int rows;

int AnzahlP = 3000;     // Anzahl der Partikel

Particle[] particles = new Particle[AnzahlP];
PVector[] flowField;

boolean record;

void setup() {
  //fullScreen();  //wenn mit deiner Webcam möglich, bitte fullScreen benutzen bzw max. Auflösung
  size(640, 480);
  background(0,50,100);

  video = new Capture(this, 640, 480, 30);    // Capturefunktion wird aufgerufen, Video wird gestartet und Bild erstellt
  video.start();
  prevFrame = createImage(video.width, video.height, RGB);

  cols = floor(width/scl);    // Definition der Größe der Rasterelemente des Flowfields
  rows = floor(height/scl);

  flowField = new PVector[(cols*rows)];    

  for (int i = 0; i < AnzahlP; i++) {    // Partikel Array
    particles[i] = new Particle();
  }
}

void draw() {

  if (video.available()) {    // Video wird im draw loop abgerufen, gezeigt und aktualisiert


    prevFrame.copy(video, 0, 0, video.width, video.height, 0, 0, width, height);

    prevFrame.updatePixels();
    prevFrame.loadPixels();

    video.read();
    video.loadPixels();
  }


  Mx = 0;
  My = 0;
  ave = 0;


  for (int xx = 0; xx < video.width; xx ++ ) {    // for-Schleifen, die average Motion errechnen, indem sie den Farbunterschied (RGB) von zwei Pixel nach Update vergleichen
    for (int yy = 0; yy < video.height; yy ++ ) {    // überschreitet dieser Unterschied den Wert von 50, wird dieser Pixel als bewegt wahrgenommen
                                                        // der average Spot aller bewegten Pixel wird errechnet und als der Punkt definiert, der durch seinen Standort die spätere Berrechnung der Patikelbewegung ermöglicht
      int loc = xx + yy*video.width;            
      color current = video.pixels[loc];      
      color previous = prevFrame.pixels[loc]; 


      float r1 = red(current); 
      float g1 = green(current); 
      float b1 = blue(current);
      float r2 = red(previous); 
      float g2 = green(previous); 
      float b2 = blue(previous);
      float diff = dist(r1, g1, b1, r2, g2, b2);    // Unterschied zwischen Farbwert des Pixels vor und nach dem Update wird berrechnet


      if (diff > threshold) {     // wenn > 50 = Bewegung = Ball wird bewegt und löst entsprechende Bewegung der Partikel aus
        pixels[loc] = video.pixels[loc];
        Mx += xx;
        My += yy;
        ave++;
      } else {
      }
    }
  }


  if (ave != 0) {     // je nach Staärke des errechneten Farbunterschieds wird Ball bewegt
    Mx = Mx/ave;
    My = My/ave;
  }
  if (Mx > ballX + rsp/2 && Mx > 50) {
    ballX+= rsp;
  } else if (Mx < ballX - rsp/2 && Mx > 50) {
    ballX-= rsp;
  }
  if (My > ballY + rsp/2 && My > 50) {
    ballY+= rsp;
  } else if (My < ballY - rsp/2 && My > 50) {
    ballY-= rsp;
  }

  // updatePixels();    // durch Herausnehmen wird Video von Sketch wahrgenommen und ausgewertet aber nicht gezeigt
  noStroke();
  fill(200, 0, 0, 0);
  ellipse(ballX, ballY, 20, 20);    // Ellipse an der Stelle an der der Average Motion Hotspot errechnet wird




  noStroke();    // Rect mit hoher Transperenz lässt alte Partikel-Standorte ausfaden
  fill(255, 255, 255, 1);
  rect(0, 0, width, height);

  float yoff = 0;
  for (int y = 0; y < rows; y++) {
    float xoff = 0;
    for (int x = 0; x < cols; x++) {
      int index = (x + y * cols);


      float a = dist(ballX, height/2, ballX, ballY);    // Größen des Dreiecks, die durch Ball und Mittelpunkt entstehen, werden berrechnet um mit Hilfe des Satz des Pythagoras letztlich den Winkel zu berrechnen 
      float b = dist(width/2, height/2, ballX, ballY);    //  der darüber entscheidet in welchem der abgesteckten Felder sich die Average Motion befindet und wie sich die Partikel enstprechend zu bewegen haben
      
      if (ballY > height/2) {
        b = b* -1;
      }
      float w = a/b;
      float m = asin(w)*-2;
      
      if (ballX < width/2 && ballY < 0.33*height) {    // hier werden die Felder abgesteckt. wenn average motion darin lokalisiert wird, entscheiden die Funktionen über die Bewegung des Partikelschwarms
        m = -5.5;
      } else if (ballX < width/2 && ballY < 0.66*height && ballY > 0.33*height) {
        m = 6.0;
      } else if (ballX < width/2 && ballY > 0.66*height) {
        m= 5.5;
      }
      
      float angle = noise(xoff, yoff, zoff) * m ;  // Richtung des Partikelschwarms wird errechnet, NoiseFunktion hält die Partikelreihe beisammen und ergibt zusammenhängenden Strom
      PVector v = PVector.fromAngle(angle);

      // println (m); 


      v.setMag(1);

      flowField[index] = v;

      stroke(0, 50);

      xoff += inc;
    }
    yoff += inc;
  }
  zoff += 0.0003;    // Stärke des Flowfields

  for (int i = 0; i < particles.length; i++) {    // auf jeden Partikel im Array werden definierten Funktionen angewendet 
    particles[i].follow(flowField);
    particles[i].update();
    particles[i].edges();
    particles[i].show();
  }
  flipHalf();

  if (record) {    // Dokumentationshilfe für Screenshot
    saveFrame("line-######.png");
  }

  if (record) {
    endRecord();
    record = false;
  }
}



void flipHalf() {    // Def. der Spiegelfunktion
  loadPixels();
  for (int x=width/2; x<width; x++) {
    for (int y=0; y<height; y++) {
      pixels[x+y*width] = pixels[(width-x)+y*width];
    }
  }

  updatePixels();
}

void mousePressed() {
  record = true;
}
